unit NRegView;

interface

uses
  Windows, Forms, Messages, SysUtils, Classes, Controls, ComCtrls, Registry,
  Dialogs, Math;

Const
     AHKey : Array [1..5] of String = ('HKEY_CLASSES_ROOT',
                                       'HKEY_CURRENT_CONFIG',
                                       'HKEY_CURRENT_USER',
                                       'HKEY_LOCAL_MACHINE',
                                       'HKEY_USERS');

type
  TRegistryType = (rtClasses_Root,
                   rtCurrent_Config,
                   rtCurrent_User,
                   rtLocal_Machine,
                   rtUsers);

  TNRegView = class(TCustomTreeView)
  private
    { Private declarations }
    FKeyNames: TStringList;
    FKeyTypes: TStringList;
    FKeyLengths: TStringList;
    FCNode: TTreeNode;
    SNode: TTreeNode;
    FCPath: String;
    FRootKey: HKEY;
    FRootPath: String;
    FRegType: TRegistryType;
    function GetPath(Node: TTreeNode): String;
    procedure SearchNames(Node: TTreeNode);
    function GetRelPath(Path: String): String;
    procedure SetRegType(Value: TRegistryType);
    procedure SearchReg(APath: String; SNode: TTreeNode; MaxLevel: Integer);
  protected
    { Protected declarations }
    procedure Change(Node: TTreeNode); override;
    procedure Expand(Node: TTreeNode); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Search;
  published
    { Published declarations }
    property RegistryType: TRegistryType read FRegType write SetRegType default rtLocal_Machine;
    property CurrentNode: TTreeNode read FCNode;
    property CurrentPath: String read FCPath;
    property RootKey: HKEY read FRootKey;
    property RootPath: String read FRootPath;
    property KeyNames: TStringList read FKeyNames;
    property KeyTypes: TStringList read FKeyTypes;
    property KeyLengths: TStringList read FKeyLengths;
    property Images;
    property ReadOnly default True;
    property ShowButtons;
    property ShowLines;
    property ShowRoot;
    property SortType default stText;
    property AutoExpand;
    property HotTrack default True;
    property RightClickSelect;
    property OnAddition;
    property OnChange;
    property OnChanging;
    property OnCollapsed;
    property OnCollapsing;
    property OnCompare;
    property OnExpanded;
    property OnExpanding;
    property Items;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TNRegView]);
end;

constructor TNRegView.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     FKeyNames:= TStringList.Create;
     FKeyTypes:= TStringList.Create;
     FKeyLengths:= TStringList.Create;
     FRegType:= rtLocal_Machine;
     FRootKey:= HKEY_LOCAL_MACHINE;
     FRootPath:= 'HKEY_LOCAL_MACHINE';
     ReadOnly:= True;
     SortType:= stText;
     HotTrack:= True;
end;

destructor TNRegView.Destroy;
begin
     FKeyLengths.Free;
     FKeyTypes.Free;
     FKeyNames.Free;
     inherited Destroy;
end;

procedure TNRegView.Search;
Begin
     FKeyNames.Clear;
     FKeyTypes.Clear;
     FKeyLengths.Clear;
     SearchReg('',Nil,2);
end;

procedure TNRegView.SetRegType(Value: TRegistryType);
begin
     If Value<>FRegType then begin
     FRegType:= Value;
     Case Value Of
          rtLocal_Machine    : Begin
                               FRootKey:= HKEY_LOCAL_MACHINE;
                               FRootPath:= 'HKEY_LOCAL_MACHINE';
                               end;
          rtClasses_Root     : Begin
                               FRootKey:= HKEY_CLASSES_ROOT;
                               FRootPath:= 'HKEY_CLASSES_ROOT';
                               end;
          rtCurrent_Config   : Begin
                               FRootKey:= HKEY_CURRENT_CONFIG;
                               FRootPath:= 'HKEY_CURRENT_CONFIG';
                               end;
          rtCurrent_User     : Begin
                               FRootKey:= HKEY_CURRENT_USER;
                               FRootPath:= 'HKEY_CURRENT_USER';
                               end;
          rtUsers            : Begin
                               FRootKey:= HKEY_USERS;
                               FRootPath:= 'HKEY_USERS';
                               end;
     end;
     end;
end;

function TNRegView.GetPath(Node: TTreeNode): String;
Var
   CPath : String;
   CNode : TTreeNode;
begin
     If Node<>Nil then Begin
        CNode:= Node;
        CPath:= CNode.Text;
        While CNode.Level<>0 do begin
              CNode:= CNode.Parent;
              CPath:= CNode.Text+'\'+CPath;
        end;
        Result:= CPath;
     end else Result:= '';
end;

function TNRegView.GetRelPath(Path: String): String;
Var
   J,I     : Integer;
   RelPath : String;
begin
     RelPath:= Path;
     For J:= 1 to 5 do Begin
         I:= Pos(AHKey[J],RelPath);
         If I<>0 then Begin
            System.Delete(RelPath,I,length(AHKey[J]));
            If length(RelPath)<>0 then While RelPath[1]='\' do System.Delete(RelPath,1,1);
         end;
     end;
     RelPath:= Trim(RelPath);
     Result:= RelPath;
end;

procedure TNRegView.Change(Node: TTreeNode);
Var
   RelPath                : String;
begin
     SNode:= Node;
     FCNode:= Node;
     FCPath:= GetPath(Node);
     SearchNames(Node);

     inherited Change(Node);

     If (not SNode.HasChildren) then
     Begin
        RelPath:= GetRelPath(FCPath);
        SearchReg(RelPath,SNode,SNode.Level+1);
     end;
end;

procedure TNRegView.Expand(Node: TTreeNode);
begin
     inherited Expand(Node);
     Node.Selected:= True;
     Change(Node);
end;

procedure TNRegView.SearchReg(APath: String; SNode: TTreeNode; MaxLevel: Integer);
var
   LReg     : TRegistry;
   Ok       : Boolean;
   I        : Integer;
   KList    : TStringList;
   RK       : String;
   LNode,
   ANode    : TTreeNode;
   ACursor  : TCursor;
   LPath    : String;
begin
     {Initialize}
     ACursor:= Screen.Cursor;
     Screen.Cursor:= crHourGlass;
     LNode:= SNode;
     If (LNode=Nil) or (LNode.Level<MaxLevel) then Begin
     LReg:= TRegistry.Create;
     LReg.Access:= KEY_READ;
     KList:= TStringList.Create;
     KList.Sorted:= True;
     try
     try
     LReg.RootKey:= FRootKey;
     LPath:= GetRelPath(GetPath(SNode));
     Ok:= LReg.OpenKeyReadOnly(LPath);
     If Ok then begin
        If LNode=Nil then Begin
           If LPath='' then
           Case FRootKey Of
           HKEY_CLASSES_ROOT   : RK:= AHKey[1];
           HKEY_CURRENT_CONFIG : RK:= AHKey[2];
           HKEY_CURRENT_USER   : RK:= AHKey[3];
           HKEY_LOCAL_MACHINE  : RK:= AHKey[4];
           HKEY_USERS          : RK:= AHKey[5];
           end;
           LNode:= Items.Add(Nil,RK);
           LNode.ImageIndex:= 0;
           LNode.SelectedIndex:= 1;
        end;
        If LReg.HasSubKeys and (LNode.Count=0)
           then LReg.GetKeyNames(KList);
     end;
     LReg.CloseKey;
     finally
            LReg.Free;
     end;
     If LNode.Count>0 then Begin
        For I:= 0 to LNode.Count-1 do Begin
            SearchReg(LPath+'\'+LNode.Item[I].Text,LNode.Item[I],MaxLevel);
        end;
     end else If (KList.Count>0) then
     For I:= 0 to KList.Count-1 do Begin
         If KList.Strings[I]<>'' then begin
            ANode:= Items.AddChild(LNode,KList.Strings[I]);
            ANode.ImageIndex:= 0;
            ANode.SelectedIndex:= 1;
            SearchReg(LPath+'\'+KList.Strings[I],ANode,MaxLevel);
         end;
     end;
     finally
            KList.Free;
     end;
     end;
     Screen.Cursor:= ACursor;
end;

procedure TNRegView.SearchNames(Node: TTreeNode);
Var
   Path         : String;
   D            : String;
   Root,I,J,L   : Integer;
   LReg         : TRegistry;
   ValSorted    : TStringList;
   DataType     : TRegDataType;
   DataLen      : Integer;
   DataVal      : Integer;
   DataStr      : String;
   DataTypeStr  : String;
   B            : Array of Byte;
   AllCh        : Boolean;
begin
        Path:= GetRelPath(GetPath(Node));
        Root:= FRootKey;
        LReg:= TRegistry.Create;
        ValSorted:= TStringList.Create;
        try
           LReg.RootKey:= Root;
           LReg.OpenKeyReadOnly(Path);
           LReg.GetValueNames(ValSorted);
           ValSorted.CaseSensitive:= True;
           ValSorted.Duplicates:= dupIgnore;
           ValSorted.Sorted:= True;
           FKeyNames.Text:= ValSorted.Text;
           FKeyTypes.Clear;
           FKeyLengths.Clear;
           For I:= 0 to FKeyNames.Count-1 do Begin
               DataType:= LReg.GetDataType(FKeyNames[I]);
               DataLen:= LReg.GetDataSize(FKeyNames[I]);
               D:= IntToStr(Integer(DataType));
               Case DaTaType Of
                    rdString,
                    rdExpandString   : Begin
                                            If DataType=rdString then DataTypeStr:= 'String'
                                                                 else DataTypeStr:= 'StringEx';
                                            try
                                            DataStr:= LReg.ReadString(FKeyNames[I]);
                                            except
                                                  On E: Exception do DataStr:= '';
                                            end;
                                       end;
                    rdInteger        : Begin
                                            DataTypeStr:= 'DWord';
                                            try
                                            DataVal:= LReg.ReadInteger(FKeyNames[I]);
                                            DataStr:= Format('0x%8.8x',[DataVal]);
                                            except
                                                  On E: Exception do DataStr:= '00000000';
                                            end;
                                       end;
                    rdBinary         : Begin
                                            DataTypeStr:= 'Binary';
                                            SetLength(B,DataLen);
                                            try
                                            try
                                            L:= LReg.ReadBinaryData(FKeyNames[I],B[0],DataLen);
                                            DataStr:= '';
                                            If (L=DataLen) and (L>0) then begin
                                            For J:= 0 to Min(L-1,16) do Begin
                                                DataStr:= DataStr+Format('%2.2x ',[B[J]]);
                                            end;
                                            DataStr:= Trim(DataStr)+'...';
                                            end;
                                            except
                                                  On E: Exception do
                                                     DataStr:= '????(Binary Data)';
                                            end;
                                            finally
                                                   B:= Nil;
                                            end;
                                       end;
                    else {rdUnknown}   Begin
                                            DataTypeStr:= 'Binary/Other';
                                            AllCh:= False;
                                            SetLength(B,DataLen);
                                            try
                                            try
                                            L:= LReg.ReadBinaryData(FKeyNames[I],B[0],DataLen);
                                            DataStr:= '';
                                            If (L=DataLen) and (L>0) then begin
                                               SetLength(B,Min(33,L+1));
                                               B[Min(32,L)]:= 0;
                                               For J:= 0 to Min(31,L-1) do Begin
                                                   AllCh:= B[J] in [32..126];
                                                   If (not AllCh) or (J>(Min(31,L-1) div 2)) then Break;
                                               end;
                                               If AllCh then Begin
                                                             DataStr:= DataStr+String(PChar(B));
                                                             If L>32 then DataStr:= DataStr+'...';
                                                        end else Begin
                                                             For J:= 0 to Min(32,L) do Begin
                                                             DataStr:= DataStr+Format('%2.2x ',[B[J]]);
                                                             end;
                                                             DataStr:= Trim(DataStr)+'...';
                                                        end;
                                            end;
                                            except
                                                  On E: Exception do
                                                     DataStr:= '????(Binary Data)';
                                            end;
                                            finally
                                                   B:= Nil;
                                            end;
                                       end;
               end;
               If FKeyNames[I]='' then FKeyNames[I]:= '{default}';
               FKeyLengths.Add(FKeyNames[I]+'='+IntToStr(DataLen));
               FKeyTypes.Add(FKeyNames[I]+'='+DataTypeStr);
               FKeyNames[I]:= FKeyNames[I]+'='+DataStr;
           end;
        finally
               LReg.Free;
               ValSorted.Free;
        end;
end;

end.
